/**
 * @module dixim
 * @submodule common
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule api-client
 */
(function() {
var ApiClient;
var prototype;
var superConstructor;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var ApiClientSession = dixim.common.ApiClientSession;
var $ = dixim.ctt.jQuery;
var Utils = dixim.ctt.util.Utils;

/**
 * Provides interfaces to transfer API.
 *
 * This API client supports two type of event handling method,
 * callback and event-subscription.
 *
 * The callback method can be specified as the parameter of each method.
 * (Such as request.success, request.error or request.complete.)
 * When the callback is called, an Event instance which is related to
 * the triggered method is passed to the callback.
 *
 * This API client is also an EventTarget, you can subscribe an event
 * by addEventListener().
 *
 * @class ApiClient
 * @constructor
 * @extends EventTarget
 * @param {Object} [configs]
 */
ApiClient = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ApiClient, EventTarget);
dixim.common.ApiClient = ApiClient;
prototype = ApiClient.prototype;
superConstructor = ApiClient.superConstructor;

var apiClientSessionManager = dixim.common.apiClientSessionManager;

var IP_VERSION = {
    "4": 4,
    "6": 6,
};

/**
 * @method _initialize
 * @protected
 * @param {Object} [configs]
 */
prototype._initialize = function(configs) {
    var isCancelAll;

    configs = configs || {};

    isCancelAll = (typeof configs.isCancelAll !== 'undefined') ? configs.isCancelAll : false;
    if (isCancelAll) apiClientSessionManager.cancelAll();

    this._ipVersion = Utils.getIpVersion();
};

var _makeResponseHandler = function(handler, args) {
    var slice = Array.prototype.slice;

    return dixim.proxy(function() {
        handler.apply(this, slice.call(arguments, 0).concat(args));
    }, this);
};

var _handleComplete = function(session, event, request) {
    if (request.complete) {
        request.complete(event);
    }
    session.connection = null;
};

var _handleFailureResponse = function(jqXhr, textStatus, errorThrown, session) {
    var event;
    var request = session.request || {};
    var data;

    if (session.isCancelled()) return;

    if ((jqXhr.responseType === 'text' || jqXhr.responseType === '') && !!jqXhr.responseText && jqXhr.responseText.match(/^{/g) && jqXhr.responseText.match(/}$/g)) {
        data = $.parseJSON(jqXhr.responseText);
    } else {
        data = null;
    }

    event = new Event(session.api + 'Error', {session: session, errorCode: jqXhr.status, data: data});
    if (request.error) {
        request.error(event);
    }
    this.dispatchEvent(event);
    _handleComplete.call(this, session, event, request);
};

var _handleSuccessResponse = function(data, textStatus, jqXhr, session) {
    var event;
    var request = session.request || {};

    if (session.isCancelled()) return;

    data = $.extend(true, {}, data, {session: session});
    event = new Event(session.api, data);
    if (request.success) {
        request.success(event);
    }
    this.dispatchEvent(event);
    _handleComplete.call(this, session, event, request);
};

var _makeAjaxParams = function(method, query, handlerArgs) {
    var asyncArg = true;

    if (handlerArgs[0].request !== null && typeof handlerArgs[0].request !== 'undefined' && typeof handlerArgs[0].request.async !== 'undefined') {
        asyncArg = handlerArgs[0].request.async;
    }
    return {
        type: method.toUpperCase(),
        data: query,
        dataType: 'json',
        cache: false,
        success: _makeResponseHandler.call(this, _handleSuccessResponse, handlerArgs),
        error: _makeResponseHandler.call(this, _handleFailureResponse, handlerArgs),
        context: this,
        async: asyncArg,
    };
};

var _encodeHTMLForm = function(data) {
    var params = [];

    for (var name in data) {
        var value = data[name];
        var param = [encodeURIComponent(name), '=', encodeURIComponent(value)].join('');

        params.push( param );
    }

    return params.join( '&' ).replace(/%20/g, '+');
};

var _ajaxForIpv6 = function(api, cfg, session) {
    var req;
    var isAutoNoCache = ((Utils.isUserAgent('chrome') || Utils.isUserAgent('firefox') || Utils.isUserAgent('safari')) && (!Utils.isUserAgent('edge')));

    if (window.XMLHttpRequest) {
        req = new XMLHttpRequest();
    } else if (window.ActiveXObject) {
        req = new ActiveXObject("Microsoft.XMLHTTP");
    } else {
        return req;
    }

    req.onreadystatechange = function() {
        var json = '';

        if (req.readyState == 4) {
            if (req.status == 200) {
                if (req.responseText !== '') json = JSON.parse(req.responseText.replace(/\\'/g, "'"));
                cfg.success(json, req.statusText, req, session);
            } else {
                cfg.error(req, req.statusText, null, session);
            }
        }
    }
    if (cfg.type.toLowerCase() === 'get') api = [api, '?', _encodeHTMLForm(cfg.data)].join('');
    req.open(cfg.type, api, cfg.async);
    if (cfg.type.toLowerCase() === 'post') req.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    if (!!!isAutoNoCache) {
        req.setRequestHeader('Cache-Control', 'no-cache');
        req.setRequestHeader('Pragma', 'no-cache');
        req.setRequestHeader('If-Modified-Since', 'Thu, 01 Jun 1970 00:00:00 GMT');
    }
    req.send(_encodeHTMLForm(cfg.data));
    return req;
};

prototype.doSimpleFetch = function(event, api, request, method) {
    var session;
    var cfg;
    var async = true;

    if(typeof method === 'undefined') method = 'GET';

    if (typeof request !== 'undefined' && typeof request.async !== 'undefined') {
        async = request.async;
    }

    $.support.cors = true;
    session = new ApiClientSession(event);
    session.request = $.extend(true, {}, request);
    cfg = _makeAjaxParams.call(this, method, null, [session]);

    if (this._ipVersion === IP_VERSION['6']) {
        session.connection = _ajaxForIpv6(api, cfg, session);
    } else {
        session.connection = $.ajax(api, cfg);
    }

    apiClientSessionManager.add(session);
    return session;
};

prototype.doSimpleMod = function(event, api, query, request, method) {
    var session;
    var cfg;
    var async = true;

    if(typeof method === 'undefined') method = 'GET';

    if (typeof request !== 'undefined' && typeof request.async !== 'undefined') {
        async = request.async;
    }

    $.support.cors = true;
    session = new ApiClientSession(event);
    session.request = $.extend(true, {}, request);
    cfg = _makeAjaxParams.call(this, method, query, [session]);
    if (request !== null && typeof request !== 'undefined' && typeof request.dataType !== 'undefined') {
        cfg.dataType = request.dataType;
    }

    if (this._ipVersion === IP_VERSION['6']) {
        session.connection = _ajaxForIpv6(api, cfg, session);
    } else {
        session.connection = $.ajax(api, cfg);
    }

    apiClientSessionManager.add(session);
    return session;
};

}());
