/**
 * @module ctt
 * @submodule content-listing-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var ContentListingView = dixim.ctt.ContentListingView;
var ContentsTransferApiClient = dixim.ctt.ContentsTransferApiClient;

var ContentListingController;
var ContentListingControllerCriteria;
var prototype;

/**
 * Represents the current criteria that is used by
 * the content list for listing items.
 *
 * @class ContentListingControllerCriteria
 * @constructor
 * @param {Object} [configs]
 * @param {String} [configs.device]
 * @param {String} [configs.containerObject]
 * @param {String} [configs.parentObject]
 * @param {String} [configs.type=browse] 'browse', 'search'
 * @param {String} [configs.filter]
 * @param {Object[]} [configs.sortCriteria] an array of {key: 'xxx', order: true|false}. Currently, 'title' can be specified as the key.
 * @param {Object} [configs.searchCriteria] {key: value} Currently, 'title' can be specified as the key.
 */
ContentListingControllerCriteria = function(configs) {
  configs = configs || {};

  /**
   * An object represents a device that is currently browsed/searched.
   * @property device
   * @type {Object}
   */
  this.device = configs.device;

  /**
   * An object represents a container that is currently browsed/searched.
   * @property containerObject
   * @type {Object}
   */
  this.containerObject = configs.containerObject;

  /**
   * An object represents the parent object of the container.
   * @property parentObject
   * @type {Object}
   */
  this.parentObject = configs.parentObject;

  /**
   * Listing type.
   *
   * @property type
   * @type {String} 'browse' or 'search'
   * @default 'browse'
   */
  this.type = configs.type || 'browse';

  /**
   * Filter string. (container|item)
   *
   * @property filter
   * @type {String}
   */
  this.filter = configs.filter;

  /**
   * @property sortCriteria
   * @type {Object[]}
   */
  this.sortCriteria = configs.sortCriteria;

  /**
   * @property searchCriteria
   * @type {Object}
   */
  this.searchCriteria = configs.searchCriteria;

  /**
   * @property totalMatches
   * @type {Number}
   */
  this.totalMatches = undefined;

  /**
   * @property startingIndex
   * @type {Number}
   */
  this.startingIndex = undefined;

  /**
   * @property requestedCount
   * @type {Number}
   */
  this.requestedCount = undefined;
};

ContentListingControllerCriteria.prototype.clone = function() {
  return dixim.extend(new ContentListingControllerCriteria(), this, true);
};

/**
 * Resets count values represent the range of the list.
 *
 * totalMatches, startingIndex and requestedCount are
 * reset to undefined.
 *
 * @method resetRangeCounts
 */
ContentListingControllerCriteria.prototype.resetRangeCounts = function() {
  delete this.totalMatches;
  delete this.startingIndex;
  delete this.requestedCount;
};

dixim.ctt.ContentListingControllerCriteria = ContentListingControllerCriteria;

/**
 * Controls content list.
 *
 * @class ContentListingController
 * @constructor
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {Boolean} [configs.isRemote=false]
 * @param {ContentsTransferApiClient} [configs.model]
 */
ContentListingController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ContentListingController, EventTarget);
dixim.ctt.ContentListingController = ContentListingController;
prototype = ContentListingController.prototype;

/**
 * @property REQUESTED_COUNT
 * @static
 * @type {Number}
 * @default 20
 */
var REQUEST_COUNT = 20;
ContentListingController.REQUEST_COUNT = REQUEST_COUNT;

/**
 * Triggered when the Transfer button has been clicked.
 *
 * @event requestForTransferring
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object[]} event.args.items
 */
var REQUEST_FOR_TRANSFERRING = 'requestForTransferring';

/**
 * Triggered when the Download button has been clicked.
 *
 * @event requestForDownload
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object[]} event.args.items
 * @param {Object} event.args.device
 */
var REQUEST_FOR_DOWNLOAD = 'requestForDownload';

/**
 * Triggered when the rename button was clicked.
 *
 * @event rename
 */

/**
 * Triggered when the delete button was clicked.
 *
 * @event delete
 */

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {Boolean} [configs.isRemote=false]
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  this._containerNode = configs.containerNode;
  this._criteria = null;
  this._session = null;
  this._isRemote = !!configs.isRemote;
  this._isAutoDelete = !!configs.isAutoDelete;
  this._isCompression = !!configs.isCompression;

  this._view = new ContentListingView({
    containerNode: this._containerNode,
    isRemote: this._isRemote,
    isAutoDelete: this._isAutoDelete,
    isCompression: this._isCompression
  });
  this._model = configs.model || new ContentsTransferApiClient();
};

/**
 * @method start
 */
prototype.start = function() {
  if (this._isRemote) {
    this._model.addEventListener('dnBrowse', this._didBrowse, this);
    this._view.addEventListener('download', this._didClickDownload, this);
  } else {
    this._model.addEventListener('browse', this._didBrowse, this);
    this._model.addEventListener('search', this._didSearch, this);
    this._view.addEventListener('rename', this._relayButtonClickEvent, this);
    this._view.addEventListener('delete', this._relayButtonClickEvent, this);
    this._view.addEventListener('lock', this._relayButtonClickEvent, this);
    this._view.addEventListener('unlock', this._relayButtonClickEvent, this);
    this._view.addEventListener('autodelete', this._relayButtonClickEvent, this);
    this._view.addEventListener('compression', this._relayButtonClickEvent, this);
    this._view.addEventListener('transfer', this._didClickTransfer, this);
  }

  this._model.addEventListener('changePin', this._didChangePin, this);

  this._view.addEventListener('scrollToTheEnd', this._didScrollToTheEnd, this);
  this._view.addEventListener('selectAll', this._didClickSelectAll, this);
  this._view.addEventListener('unselectAll', this._didClickUnselectAll, this);
  this._view.addEventListener('clickItem', this._didClickItem, this);
  this._view.addEventListener('requestChangeSortCriteria', this._didRequestChangeSortCriteria, this);
  this._view.show();
};

prototype._relayButtonClickEvent = function(e) {
  this.dispatchEvent(new Event(e.type, e.args, e.userData));
};

prototype._didChangePin = function(e) {
  this.pin = e.args.pin;
  this._reload();
};

prototype._didBrowse = function(e) {
  this._didFetchContentList(e);
};

prototype._didSearch = function(e) {
  this._didFetchContentList(e);
};

prototype._isRootContainer = function(container) {
  if (this._isRemote) {
    return container.id === '0';
  } else {
    return container.id === 'FS-Folder' ||
           container.id === this._criteria.device.rootContainerId;
  }
};

prototype._didFetchContentList = function(e) {
  var args = e.args;
  var data = args.data;
  var results = data.results;
  var session = args.session;
  var request = session.request;
  var cri = this._criteria;
  var v = this._view;

  if (session !== this._session) return;

  if (request.browseFlag === 'metadata') {
    cri.parentObject = results[0];
    this._session = null;
    this._fetchData();
    return;
  }

  if (this._isRootContainer(cri.containerObject)) {
    v.disableStaticRow();
  } else {
    v.enableStaticRow();
  }

  cri.totalMatches = data.totalMatches;
  cri.startingIndex = request.startingIndex;
  cri.requestedCount = data.requestedCount;
  v.addRows(results);

  this._session = null;
};

prototype._didScrollToTheEnd = function(e) {
  this._fetchData();
};

prototype._didClickSelectAll = function(e) {
  this._view.selectAllRows();
};

prototype._didClickUnselectAll = function(e) {
  this._view.unselectAllRows();
};

prototype._didClickItem = function(e) {
  var data = e.args.data;
  var type = data['class'];
  var parent = null;
  var cc;
  var container = null;
  var device = null;

  if (!!(cc = this._criteria)) {
    parent = cc.parentObject;
    device = cc.device;
  }

  if (type === 'parentContainer' && parent) {
    container = parent;
  } else if (type === 'container'){
    container = data;
  } else {
    return;
  }

  this._updateCurrentCriteria(new ContentListingControllerCriteria({
    containerObject: container,
    device: device
  }));
};

prototype._didClickTransfer = function(e) {
  var args = {};

  args.items = this._view.getSelectedRows();
  this.dispatchEvent(new Event(REQUEST_FOR_TRANSFERRING, args));
};

prototype._didClickDownload = function(e) {
  var args = {};

  args.items = this._view.getSelectedRows();
  if (!!this._criteria) {
    args.device = this._criteria.device;
  }
  this.dispatchEvent(new Event(REQUEST_FOR_DOWNLOAD, args));
};

prototype._didRequestChangeSortCriteria = function(e) {
  var args = e.args;
  var sortCriteria = args.criteria;
  var cri = this._criteria;
  var nc;

  if (!!!cri) return;

  nc = cri.clone();
  nc.sortCriteria = !!sortCriteria ? sortCriteria : [];
  nc.resetRangeCounts();
  this._updateCurrentCriteria(nc);
};

/**
 * @method updateCurrentCriteria
 * @param {ContentListingControllerCriteria} criteria
 */
prototype.updateCurrentCriteria = function(criteria) {
  this._updateCurrentCriteria(criteria);
};

prototype._updateCurrentCriteria = function(criteria) {
  this._invalidateSession();
  this._view.removeAllRows();
  this._view.setSortCriteria(criteria.sortCriteria);
  this._criteria = criteria;
};

prototype._fetchData = function() {
  var cri = this._criteria;
  var m = this._model;
  var container;
  var device;
  var tm;
  var start;

  if (!!this._session) return;
  if (!!!cri) return;

  container = cri.containerObject;

  if (cri.type === 'browse' && !!!cri.parentObject && !this._isRootContainer(container)) {
    this._fetchParentData();
    return;
  }

  tm = cri.totalMatches;
  start = this._view.getNumberOfRows();
  if (typeof tm !== 'undefined' && (tm <= start)) {
    return;
  }

  if (cri.type === 'search') {
    this._session = m.search({
      id: container.id,
      startingIndex: start,
      requestedCount: REQUEST_COUNT,
      sortCriteria: cri.sortCriteria,
      searchCriteria: cri.searchCriteria
    });
  } else {
    device = cri.device;
    this._session = this._browseImpl({
      id: container.id,
      udn: !!device ? device.id : undefined,
      startingIndex: start,
      requestedCount: REQUEST_COUNT,
      sortCriteria: cri.sortCriteria,
      filter: cri.filter,
      pin: cri.pin || null,
      rating: true
    });
  }
};

prototype._browseImpl = function(data) {
  var m = this._model;

  if (this._isRemote) {
    return m.dnBrowse(data);
  } else {
    if (data.id === 'FS-Folder') {
      // XXX: Returned contents of FS-Folder and device's
      //      root container Id are different. So normalizes it.
      //      However, if browse metadata, it doesn't accept device's
      //      root container ID, so skips the normalization.
      if (data.browseFlag !== 'metadata') {
        data.id = this._criteria.device.rootContainerId;
      }
      // XXX: ... end
    }
    return m.browse(data);
  }
};

prototype._fetchParentData = function() {
  var cri = this._criteria;
  var container = cri.containerObject;
  var device = cri.device;

  this._session = this._browseImpl({
    id: container.parentId,
    udn: !!device ? device.id : undefined,
    browseFlag: 'metadata'
  });
};

prototype._invalidateSession = function() {
  if (!!!this._session) return;
  this._session.cancel();
  this._session = null;
};

prototype.getSelectedItems = function() {
  return this._view.getSelectedRows();
};

/**
 * Sets PIN.
 *
 * @method setPin
 * @param {String} pin
 */
prototype.setPin = function(pin) {
  this._model.setPin(pin);
};

/**
 * Reloads current container contents.
 *
 * @method reload
 */
prototype.reload = function() {
  this._reload();
};

prototype._reload = function() {
  var cri;

  if (this._criteria) {
    cri = this._criteria.clone();
    cri.resetRangeCounts();
    this._updateCurrentCriteria(cri);
  }
};

}());
