/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule extensions-controller
 */

(function() {
var ExtensionsController;
var prototype;
var superConstructor;
var ExtensionsView = dixim.server.ExtensionsView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ExtensionsEditDialogController = dixim.server.ExtensionsEditDialogController;
var ApiClientServer = dixim.common.ApiClientServer;
var BlockUi = dixim.common.BlockUi;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var ConfirmModalDialogView = dixim.common.ConfirmModalDialogView;
var DeletionModalDialogView = dixim.common.DeletionModalDialogView;

/**
 * @class ExtensionsController
 * @constructor
 * @param {Object} configs
 */
ExtensionsController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ExtensionsController, EventTarget);
dixim.server.ExtensionsController = ExtensionsController;
prototype = ExtensionsController.prototype;
superConstructor = ExtensionsController.superConstructor;

var SELECT = 'select';
var FORMAT = 'format';
var DELETE = 'delete';
var EDIT = 'edit';
var APPEND = 'append';
var PREV = 'prev';
var OK = 'ok';
var YES = 'yes';
var COMPLETE = 'complete';
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'extensions'].join('');

    this._model = new ApiClientServer();
    this._model.addEventListener('fetchSharedType', this._successFetchSharedType, this);
    this._model.addEventListener('fetchSharedTypeError', this._errorFetchSharedType, this);
    this._model.addEventListener('removeSharedType2', this._successRemoveSharedType, this);
    this._model.addEventListener('removeSharedType2Error', this._errorRemoveSharedType, this);
    this._model.addEventListener('resetSharedType', this.successResetSharedType, this);
    this._model.addEventListener('resetSharedTypeError', this.errorResetSharedType, this);

    this._view = new ExtensionsView(configs);
    this._view.addEventListener(SELECT, this._clickSelect, this);
    this._view.addEventListener(FORMAT, this._clickFormat, this);
    this._view.addEventListener(DELETE, this._clickDelete, this);
    this._view.addEventListener(EDIT, this._clickEdit, this);
    this._view.addEventListener(APPEND, this._clickAppend, this);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._title = _('server_menu_extensions');
    this._isProcessError = false;
    this._parentContext = configs.parentContext;
    this._selectedSharedTypes = [];
    this._blockUi = new BlockUi();
    this._currentSharedTypes = [];
    this._dialog = null;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._fetchSharedType();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._fetchSharedType = function() {
    this._blockUi.block();
    this._model.fetchSharedType();
};

prototype._successFetchSharedType = function(e) {
    var sharedTypes = e.args.sharedTypes;
    var sharedTypesLen = sharedTypes.length;

    this._currentSharedTypes = sharedTypes;
    this._blockUi.unblock();
    if (sharedTypesLen === 0) {
        this._view.error(_('server_extension_nothing'));
        return;
    }
    for (var i = 0; i < sharedTypesLen; i += 1) {
        this._view.addSharedType(sharedTypes[i]);
    }
};

prototype._errorFetchSharedType = function() {
    this._blockUi.unblock();
    this._view.error(_('server_extension_error_fetch'));
};

prototype._formatSharedTypes = function(e) {
    this._blockUi.block();
    this._model.resetSharedType();
    this._dialog.hide();
};

prototype.successResetSharedType = function() {
    this._blockUi.unblock();
    this._updateExtensionList();
};

prototype.errorResetSharedType = function() {
    this._errorDialog(_('server_extension_error_format'));
    this._blockUi.unblock();
    this._updateExtensionList();
};

prototype._successRemoveSharedType = function(e) {
    this._dialog.decrementDescription();
};

prototype._errorRemoveSharedType = function(e) {
    this._view.decrementDeletionDialog();
    this._isProcessError = true;
};

prototype._completeRemoveSharedTypes = function() {
    this._updateExtensionList();
    if (this._isProcessError) {
        this._errorDialog(_('server_extension_error_remove'));
    }
};

prototype._removeSharedTypes = function(e) {
    var selectedSharedTypesLen = this._selectedSharedTypes.length;
    var query = {};

    this._isProcessError = false;
    for (var i = 0; i < selectedSharedTypesLen; i += 1) {
        query.extensions = [this._selectedSharedTypes[i].extension];
        query.mimeTypes = [this._selectedSharedTypes[i].mimeType];
        this._model.removeSharedType2(query);
    }
};

prototype._clickSelect = function() {
    this._view.switchSelect();
};

prototype._clickFormat = function() {
    var dialog;
    var query = {};

    query.id = this._id;
    query.title = this._title;
    query.description = _('server_extension_confirm_format');
    query.buttonText = _('dialog_format');
    this._dialog = new ConfirmModalDialogView(query);
    this._dialog.addEventListener(OK, this._formatSharedTypes, this);
    this._dialog.show();
};

prototype._clickDelete = function(e) {
    var sharedTypes= e.args.sharedTypes;
    var sharedTypesLen = sharedTypes.length;
    var query = {};

    this._selectedSharedTypes = $.extend(true, [], sharedTypes);
    if (sharedTypesLen === 0) {
        this._errorDialog(_('server_extension_no_select_delete'));
        return;
    }

    query.id = this._id;
    query.title = this._title;
    query.description = _('server_extension_deletion_description', sharedTypes.length);
    query.contents = sharedTypes;
    query.nameKey = 'extension';
    this._dialog = new DeletionModalDialogView(query);
    this._dialog.addEventListener(YES, this._removeSharedTypes, this);
    this._dialog.addEventListener(COMPLETE, this._completeRemoveSharedTypes, this);
    this._dialog.show();
};

prototype._clickEdit = function(e) {
    var sharedTypes= e.args.sharedTypes;
    var sharedTypesLen = sharedTypes.length;
    var controller;
    var query = {};

    this._selectedSharedTypes = sharedTypes;
    if (sharedTypesLen !== 1) {
        this._errorDialog(_('server_extension_no_select_edit'));
        return;
    }

    query.sharedTypes = this._currentSharedTypes;
    query.itemData = sharedTypes[0];
    controller = new ExtensionsEditDialogController(query);
    controller.addEventListener(CLOSE, this._updateExtensionList, this);
    controller.start();
};

prototype._clickAppend = function() {
    var controller;
    var query = {};

    query.sharedTypes = this._currentSharedTypes;
    controller = new ExtensionsEditDialogController(query);
    controller.addEventListener(CLOSE, this._updateExtensionList, this);
    controller.start();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._updateExtensionList = function() {
    this._view.cleanExtensionList();
    this._fetchSharedType();
};

prototype._errorDialog = function(text) {
    var dialog;
    var query = {};

    query.id = this._id;
    query.title = this._title;
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
